import socket
import json
import requests
import urllib3

urllib3.disable_warnings()

def fetch_json_from_socket(host, port):
    http_request = f"GET / HTTP/1.1\r\nHost: {host}\r\n\r\n"
    try:
        with socket.create_connection((host, port), timeout=5) as sock:
            sock.sendall(http_request.encode())
            response_chunks = []
            while True:
                chunk = sock.recv(4096)
                if not chunk:
                    break
                response_chunks.append(chunk)
        response = b''.join(response_chunks).decode()
        _, _, body = response.partition('\r\n\r\n')
        return json.loads(body)
    except (socket.error, json.JSONDecodeError) as e:
        print(f"Fel vid anslutning eller JSON-avkodning från {host}:{port} -> {e}")
        return None

def fetch_json_from_http_api():
    BASE_URL = "http://192.168.1.254"
    LOGIN_URL = BASE_URL
    API_URL = BASE_URL + "/data/status.client.user.json?operation=load"

    USERNAME = "timmpo"
    PASSWORD = "AC3A35B18786903EA114B40BDEAE3AB6"

    DEFAULT_HEADERS = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:139.0) Gecko/20100101 Firefox/139.0",
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Language": "en-US,en;q=0.5",
        "Accept-Encoding": "gzip, deflate",
        "X-Requested-With": "XMLHttpRequest",
        "Connection": "keep-alive",
        "Referer": BASE_URL,
        "Origin": BASE_URL,
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
    }

    session = requests.Session()

    try:
        res = session.get(BASE_URL, headers=DEFAULT_HEADERS, verify=False)
        res.raise_for_status()
        print("🟢 Initierad session.")
    except Exception as e:
        print("❌ Fel vid initiering:", e)
        return None

    payload = {"username": USERNAME, "password": PASSWORD}
    try:
        login = session.post(LOGIN_URL, headers=DEFAULT_HEADERS, data=payload, verify=False)
        login.raise_for_status()
        print("✅ Inloggning lyckades.")
    except Exception as e:
        print("❌ Inloggning misslyckades:", e)
        return None

    cookie_value = session.cookies.get("COOKIE")
    if not cookie_value:
        print("❌ Ingen COOKIE efter inloggning.")
        return None

    cookies = {"COOKIE": cookie_value}
    try:
        api = session.get(API_URL, headers=DEFAULT_HEADERS, cookies=cookies, verify=False)
        api.raise_for_status()
        print("📦 API-svar hämtat.")
        return api.json()
    except Exception as e:
        print("❌ API-anrop misslyckades:", e)
        return None

def main():
    # Lista med tuples: (server_namn, host, port)
    servers = [
        ("router_main", "192.168.1.1", 12345),
        ("router_floor_1", "192.168.1.199", 12345),
    ]

    combined_data = {}

    for name, host, port in servers:
        data = fetch_json_from_socket(host, port)
        if data is not None:
            combined_data[name] = data

    server_main_data = fetch_json_from_http_api()
    if server_main_data is not None:
        combined_data["router_outside"] = server_main_data

    if combined_data:
        with open("combined_data.json", "w", encoding="utf-8") as f:
            json.dump(combined_data, f, ensure_ascii=False, indent=2)
        print("All data sparad i combined_data.json")
    else:
        print("Ingen data att spara.")

if __name__ == "__main__":
    main()
